==> api_server_controller-0.2.3a12-py3-none-any.whl: asc/tools/marketplace/control.py <==
from typing import List
from decimal import Decimal
from sqlalchemy import Sequence

from asc import message
from asc.core import env
from asc.models.marketplace import Tax
from asc.models.marketplace import Client
from asc.core.ABC.Control import ControlTool
from asc.tools.mailer.control \
    import ControlToolHandler as CTHMailer
from asc.tools.storage.control \
    import ControlToolHandler as CTHStorage
from asc.tools.marketplace import data
from asc.tools.marketplace.mappings import PaymentMethod
from asc.tools.marketplace.mappings import PaymentGateway
from asc.tools.marketplace.mercadopago.data import Customer
from asc.tools.marketplace.mercadopago.control \
    import ControlToolHandler as CTHMercadopago
from asc.tools.marketplace.pagarme.control \
    import ControlToolHandler as CTHPagarme
from asc.exceptions.marketplace import UserDoesNotExists
from asc.exceptions.marketplace import PaymentGatewayNotDefined


class ControlToolHandler(ControlTool):

    def __init__(self):
        super(ControlToolHandler, self).__init__()
        self.reference = None
        self.payment_info = message.M_WITHOUT_PAYMENT_INFO
        self.payment_gateway: PaymentGateway = None

    def __check_payment_gateway(self):
        if not self.payment_gateway:
            raise PaymentGatewayNotDefined()

    def __sanitize_payment_value(self, amount):
        if type(amount) is int:
            amount = str(amount / 100)
        if type(amount) in [float, Decimal]:
            amount = str(amount)

        amount = amount.replace('.', ',')
        return amount

    def set_payment_info(self, payment_info: str):
        """ Preenche descrição do pagamento.
        """
        self.payment_info = payment_info

    def set_payment_gateway(self, gateway: PaymentGateway):
        """ Define gateway de pagamento a ser utilizado pelo controlador.
        :param gateway:
        :return:
        """
        self.payment_gateway = gateway

    def get_reference(self):
        """ Gera referencia para ser utilizada publicamente. """
        return Sequence('marketplace_reference_seq').next_value

    def get_payment_by_id(self, id_payment: int):
        """ Obtém pagamento da API com o gateway definido.
        :param id_payment: id do pagamento.
        :return:
        """
        self.__check_payment_gateway()

        if self.payment_gateway == PaymentGateway.mercadopago:
            mpg_control = CTHMercadopago()
            return mpg_control.get_payment(id_payment)

    def find_client_by_email(self, email: str) -> Client:
        """ Encontra cliente marketplace por email.
        :param email: Quantos dias para expirar.
        """
        return self.session \
            .query(Client) \
            .where(Client.email == email) \
            .first()

    def calc_card_installments(self, amount: float, id: int = None,
                               to_dict=False) \
            -> List[data.Installment]:
        """ Calcula parcelas para cobranças de cartão de crédito.
        :param amount: valor autorizado à cobrança.
        :param id: id do split.
        :param to_dict: retorna valores formatados como dict.
        :return:
        """
        self.__check_payment_gateway()

        installments = []
        query = self.session.query(Tax) \
            .where(
            Tax.is_active.is_(True),
            Tax.id_marketplace_payment_method == PaymentMethod.card.value,
            Tax.id_marketplace_payment_gateway == self.payment_gateway.value) \
            .first()

        i = 1

        tax_subtract = bool(env.get(
            'marketplace__installments_tax_subtract',
            'True'
        ) == 'True')

        tax_in_serie = bool(env.get(
            'marketplace__installments_tax_in_serie',
            'True'
        ) == 'True')

        for row_cust_split_amount in query.cust_split_amount:
            installment = data.Installment(
                count=i,
                total=amount,
                tax=[
                    (query.cust_process_type, query.cust_process_amount),
                    (query.cust_split_type, row_cust_split_amount)
                ],
                subtract=tax_subtract,
                in_serie=tax_in_serie
            )

            if to_dict:
                installment = installment.asdict()
                installment['price'] = int(installment['price'])
                installment['total_tax'] = int(installment['total_tax'])
                del installment['tax']

            installments.append(installment)

            i += 1

        return installments

    def create_transaction_pix(self, client: Client, amount):
        """ Cria uma transação PIX com o gateway definido.
        :param client: cliente pagador da transação.
        :param amount: valor da transação, inteira em centavos.
        :return: dict contendo dados do pagamento.
        """
        self.__check_payment_gateway()

        if not client:
            raise UserDoesNotExists(message.E_NOT_FOUND_CLIENT)

        if self.payment_gateway == PaymentGateway.pagarme:
            pagarme = CTHPagarme()
            payment = pagarme.create_transaction_pix(
                customer=Customer(
                    name=client.name,
                    email=client.email,
                    phone_numbers=[client.phone],
                    documents=[
                        {'type': 'cpf', 'number': client.document}
                    ],
                    external_id=str(client.id_marketplace_client)
                ),
                amount=self.payment_gateway.get_amount(amount),
                info=self.payment_info
            )

        storage = CTHStorage()
        storage.set_fold('payments')
        storage_data = storage.save_qrcode(
            payment.get('qr'), payment.get('id')
        )

        payment['db_payment'].id_storage = storage_data[0]
        payment['qr_link'] = storage_data[1]
        payment['expires'] = payment['expires'].strftime('%d/%m/%Y')

        return payment

    def send_transaction_pix_email(
            self,
            client: Client,
            payment: dict,
            slug: str = 'created'):
        """
        Envia email para cliente, solicitando pagamento.
        :param client: dados do cliente.
        :param payment: dados do pagamento.
        :return:
        """
        payment_value = self.__sanitize_payment_value(
            payment.get('amount')
        )

        mailer = CTHMailer()
        mailer.send_mail(
            to=client.email,
            slug=f'{env.get("marketplace__soft_descriptor")}'
                 f'-payment-pix-{slug}',
            subject=f'{message.M_SUBJECT_PAYMENT_PIX} {self.payment_info}',
            html_vars={
                'client_name': client.name,
                'payment_value': payment_value,
                'payment_qr_code': payment.get('qr'),
                'payment_qr_link': payment.get('qr_link'),
                'payment_expires': payment.get('expires'),
                'payment_description': self.payment_info,
            },
            id_client=client.id_marketplace_client
        )

        return True

==> api_server_controller-0.2.4a9-py3-none-any.whl: asc/tools/marketplace/control.py <==
from datetime import date
from datetime import datetime
from decimal import Decimal
from hashlib import sha384
from typing import List

from sqlalchemy import Sequence

from asc import message
from asc.core import env
from asc.core.ABC.Control import ControlTool
from asc.core.filters.filter_crypme import Crypme
from asc.exceptions.marketplace import PaymentGatewayNotDefined
from asc.exceptions.marketplace import UserDoesNotExists
from asc.models.marketplace import Client
from asc.models.marketplace import Tax
from asc.models.marketplace import UserCard
from asc.tools.mailer.control \
    import ControlToolHandler as CTHMailer
from asc.tools.marketplace import data
from asc.tools.marketplace.mappings import PaymentGateway
from asc.tools.marketplace.mappings import PaymentMethod
from asc.tools.marketplace.mercadopago.control \
    import ControlToolHandler as CTHMercadopago
from asc.tools.marketplace.pagarme.control \
    import ControlToolHandler as CTHPagarme
from asc.tools.marketplace.pagarme.data import Customer as PGCustomer
from asc.tools.storage.control \
    import ControlToolHandler as CTHStorage


class ControlToolHandler(ControlTool):

    def __init__(self):
        super(ControlToolHandler, self).__init__()
        self.reference = None
        self.payment_info = message.M_WITHOUT_PAYMENT_INFO
        self.payment_gateway: PaymentGateway = None

    def __check_payment_gateway(self):
        if not self.payment_gateway:
            raise PaymentGatewayNotDefined()

    def __sanitize_payment_value(self, amount):
        if type(amount) is int:
            amount = str(amount / 100)
        if type(amount) in [float, Decimal]:
            amount = str(amount)

        amount = amount.replace('.', ',')
        return amount

    def set_payment_info(self, payment_info: str):
        """ Preenche descrição do pagamento.
        """
        self.payment_info = payment_info

    def set_payment_gateway(self, gateway: PaymentGateway):
        """ Define gateway de pagamento a ser utilizado pelo controlador.
        :param gateway:
        :return:
        """
        self.payment_gateway = gateway

    def get_reference(self):
        """ Gera referencia para ser utilizada publicamente. """
        return Sequence('marketplace_reference_seq').next_value

    def get_payment_by_id(self, id_payment: int):
        """ Obtém pagamento da API com o gateway definido.
        :param id_payment: id do pagamento.
        :return:
        """
        self.__check_payment_gateway()

        if self.payment_gateway == PaymentGateway.mercadopago:
            mpg_control = CTHMercadopago()
            return mpg_control.get_payment(id_payment)

    def find_client_by_email(self, email: str) -> Client:
        """ Encontra cliente marketplace por email.
        :param email: Email do cliente a buscar.
        """
        return self.session \
            .query(Client) \
            .where(Client.email == email) \
            .first()

    def calc_card_installments(self, amount: float, id: int = None,
                               to_dict=False) \
            -> List[data.Installment]:
        """ Calcula parcelas para cobranças de cartão de crédito.
        :param amount: valor autorizado à cobrança.
        :param id: id do split.
        :param to_dict: retorna valores formatados como dict.
        :return:
        """
        self.__check_payment_gateway()

        installments = []
        query = self.session.query(Tax) \
            .where(
            Tax.is_active.is_(True),
            Tax.id_marketplace_payment_method == PaymentMethod.card.value,
            Tax.id_marketplace_payment_gateway == self.payment_gateway.value) \
            .first()

        i = 1

        tax_subtract = bool(env.get(
            'marketplace__installments_tax_subtract',
            'True'
        ) == 'True')

        tax_in_serie = bool(env.get(
            'marketplace__installments_tax_in_serie',
            'True'
        ) == 'True')

        for row_cust_split_amount in query.cust_split_amount:
            installment = data.Installment(
                count=i,
                total=amount,
                tax=[
                    (query.cust_process_type, query.cust_process_amount),
                    (query.cust_split_type, row_cust_split_amount)
                ],
                subtract=tax_subtract,
                in_serie=tax_in_serie
            )

            # Retorna quando parcela não atinge valor mínimo
            if installment.price <= 5:
                return installments

            if to_dict:
                installment = installment.asdict()
                installment['price'] = int(installment['price'])
                installment['total_tax'] = int(installment['total_tax'])
                del installment['tax']

            installments.append(installment)

            i += 1

        return installments

    def create_card_hash(self, client_card:UserCard):
        """ Cria hash unica para cartão de usuário.
        """
        hs = ''

        time = f'{datetime.today().microsecond:06}'
        timeA= time[0:3]
        timeB= time[3:6]

        hs += timeA

        parts= [
            client_card.nm_holder[0].upper(),
            client_card.dt_exp,
            f'{client_card.nr_card}{env["marketplace__card_salt_01"]}',
            f'{int(client_card.nr_cvv) + int(env["marketplace__card_salt_02"])}',
            client_card.nm_holder[-1].upper(),
        ]

        hs += sha384(''.join(parts).encode('UTF-8')).hexdigest()
        hs += timeB

        return hs

    def create_card(self, **card) -> int:
        """ Registra cartão ou, quando existente, retorna o id.
        :param card:
        :return:
        """

        client_card = UserCard()
        client_card.dt_add = date.today()
        client_card.nr_cvv = card.get('cvv')
        client_card.dt_exp = card.get('date')
        client_card.id_user = card.get('user')
        client_card.nr_card = card.get('number')
        client_card.nm_holder = card.get('holder')

        crypme = Crypme()
        crypme.getGrid(client_card.dt_add)
        client_card.nr_card = crypme.atob(client_card.nr_card)
        client_card.hs_card = self.create_card_hash(client_card)

        client_card_search = self.session.query(UserCard) \
            .where(
                UserCard.dt_exp==client_card.dt_exp,
                UserCard.nr_cvv==client_card.nr_cvv,
                UserCard.nr_card==client_card.nr_card
        ).first()

        if client_card_search:
            return client_card_search.id_user_card

        self.session.add(client_card)
        self.session.flush()
        self.session.commit()

        return client_card.id_user_card

    def create_transaction_card(self,
                                client:Client,
                                client_card:UserCard,
                                amount:int,
                                reference:str,
                                installments:int=1
                                ):
        """ Cria uma transação por cartão com o gateway definido.
        :param client: cliente pagador da transação.
        :param client_card: cartão do cliente.
        :param amount: valor da transação, inteira em centavos.
        :param reference: referencia externa.
        :param installments: número vezes a dividir o pagamento.
        :return: dict contendo dados do pagamento.
        """
        self.__check_payment_gateway()

        if not client:
            raise UserDoesNotExists(message.E_NOT_FOUND_CLIENT)

        if self.payment_gateway == PaymentGateway.mercadopago:
            #TODO: criar pagamento de cartão mercado pago

            mercadopago = CTHMercadopago()
            payment = mercadopago.send_payment(
                amount=round(amount / 100, 2),
                id_user=client.id_marketplace_client,
                id_user_card=client_card.id_user_card,
                #external_reference=str(reference),
                installments=installments,
            )

        return payment

    def create_transaction_pix(self, client: Client, amount, **props):
        """ Cria uma transação PIX com o gateway definido.
        :param client: cliente pagador da transação.
        :param amount: valor da transação, inteira em centavos.
        :return: dict contendo dados do pagamento.
        """
        self.__check_payment_gateway()

        if not client:
            raise UserDoesNotExists(message.E_NOT_FOUND_CLIENT)

        if self.payment_gateway == PaymentGateway.pagarme:
            pagarme = CTHPagarme()
            payment = pagarme.create_transaction_pix(
                customer=PGCustomer(
                    name=client.name,
                    email=client.email,
                    phone_numbers=[client.phone],
                    documents=[
                        {'type': 'cpf', 'number': client.document}
                    ],
                    external_id=str(client.id_marketplace_client)
                ),
                amount=self.payment_gateway.get_amount(amount),
                info=self.payment_info,
                **props
            )

        storage = CTHStorage()
        storage.set_fold('payments')
        storage_data = storage.save_qrcode(
            payment.get('qr'), payment.get('id')
        )

        payment['db_payment'].id_storage = storage_data[0]
        payment['qr_link'] = storage_data[1]
        payment['expires'] = payment['expires'].strftime('%d/%m/%Y')

        return payment

    def send_transaction_pix_email(
            self,
            client: Client,
            payment: dict,
            slug: str = 'created'):
        """
        Envia email para cliente, solicitando pagamento.
        :param client: dados do cliente.
        :param payment: dados do pagamento.
        :return:
        """
        payment_value = self.__sanitize_payment_value(
            payment.get('amount')
        )

        mailer = CTHMailer()
        mailer.send_mail(
            to=client.email,
            slug=f'{env.get("marketplace__soft_descriptor")}'
                 f'-payment-pix-{slug}',
            subject=f'{message.M_SUBJECT_PAYMENT_PIX} {self.payment_info}',
            html_vars={
                'client_name': client.name,
                'payment_value': payment_value,
                'payment_qr_code': payment.get('qr'),
                'payment_qr_link': payment.get('qr_link'),
                'payment_expires': payment.get('expires'),
                'payment_description': self.payment_info,
            },
            id_client=client.id_marketplace_client
        )

        return True
